unit mainFormUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ShellCtrls, Grids, Outline, DirOutln,
  Buttons, ExtCtrls, Qdialogs, FileCtrl;

type
  TmainForm = class(TForm)
    fichierSource: TEdit;
    B_selFicSource: TBitBtn;
    Label1: TLabel;
    Label2: TLabel;
    repDestPDB: TEdit;
    B_selRepDestPDB: TBitBtn;
    Label3: TLabel;
    StoryName: TEdit;
    B_transformer: TButton;
    Image1: TImage;
    OpenDialog1: TOpenDialog;
    Label4: TLabel;
    Button1: TButton;
    procedure B_selRepDestClick(Sender: TObject);
    procedure B_selFicSourceClick(Sender: TObject);
    procedure B_transformerClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);

  private
    { Dclarations prives }
  public
    { Dclarations publiques }

    Function Txt2Pdb(strText, strTitle: string; intCategory: integer) : string;
    Function FourBytes(dblNumber : longint) : string;
    Function TwoBytes(lngNumber : longint) : string;


  end;

var
  mainForm: TmainForm;

implementation

{$R *.dfm}
uses strUtils, fichierSelectUnit;


Function TmainForm.Txt2Pdb(strText, strTitle: string; intCategory: integer) : string;
var
 strIn, strOut : string;
 intRecords : integer;
 intCounter : integer;
 dblSize, dblRecordOffset, dblTime : longint;
begin
{
Simplified PDB DOC format:
'Pdb Title                32
'Pdb Header               48
'Record Offset 1           8
'Record Offset 2           8
'Record Offset N           8
'Padding                   2
'Record 1 Doc Header      16
'Record 2 Doc Data 1    4096
'Record N Doc Data N-1  4096
}



    // Title (Offset 0, Length 32)
    strOut := LeftStr(strTitle, 31) + DupeString(chr(0),32 - Length(LeftStr(strTitle, 31)));
    // Attributes (Offset 32) &H2=Read only &H8=Backup &H10=install over existing &H20=reset after install
    strOut := strOut + DupeString( Chr(0),2);
    // Version (Offset 34)
    strOut := strOut + DupeString(Chr(0),2);
    // Create Time (Offset 36) 'Seconds since Jan 1, 1970
    strOut := strOut + FourBytes(Round((date()-strTodate('01/01/1970')) * 24 * 60 * 60));
    //Modify Time (Offset 40) 'Seconds since Jan 1, 1970
    strOut := strOut + FourBytes(Round((date()-strTodate('01/01/1970')) * 24 * 60 * 60));
    //Backup Time (Offset 44) 'Zero if never backed up
    strOut := strOut + DupeString(Chr(0),4);
    //Modification Number (Offset 48)
    strOut := strOut + DupeString(Chr(0),4);
    //App Info ID (Offset 52)
    strOut := strOut + DupeString(Chr(0),4);
    //Sort Info ID (Offset 56)
    strOut := strOut + DupeString(Chr(0),4);
    //Type (Offset 60)
    strOut := strOut + 'TEXt';
    //Creator (Offset 64)
    strOut := strOut + 'REAd';
    //ID Seed (Offset 68)
    strOut := strOut + DupeString( Chr(0),4);
    //Next Record List (Offset 72)
    strOut := strOut + DupeString(Chr(0),4);
    //Number of Records (Offset 76)
    intRecords := trunc(Length(strIn) / 4096) + 2;
    strOut := strOut + TwoBytes(intRecords);
    //Record List (Offset 78) 8 bytes describing each record
    For intCounter := 1 To intRecords do
    begin
        //The first record is a 16-byte DOC header
        dblRecordOffset := 78 + 2 + (intRecords * 8);
        If intCounter > 1 Then
          begin
            //Records 2 thru N are data records
            dblRecordOffset := dblRecordOffset + 16 + (intCounter - 2) * 4096;
          end;
        strOut := strOut + FourBytes(dblRecordOffset); //Record offset
        strOut := strOut + Chr(64 Or intCategory); //delete dirty busy secret category
        strOut := strOut + 'o' + TwoBytes((intCounter - 1) Or 32768); //Unique ID
    end; // for
    //Two-byte padding
    strOut := strOut + DupeString( Chr(0),2);
    //---------- RECORD 1 (DOC HEADER) ----------
    //Version
    strOut := strOut + Chr(0) + Chr(1); //1=plain, 2=compressed
    //Reserved1
    strOut := strOut + Chr(0) + Chr(0);
    //Document Size (uncompressed)
    strOut := strOut + FourBytes(Length(strText));
    //Number of Records
    strOut := strOut + TwoBytes(intRecords - 1);
    //Record Size
    strOut := strOut + TwoBytes(4096); //'4096
    //Reserved
    strOut := strOut + DupeString( Chr(0),4);
    //---------- THE DOCUMENT ----------
    strOut := strOut + strText;

    // Enlever les #13 qui parasitent le source sur POSE
    Txt2Pdb := strOut;
End;

Function TMainForm.FourBytes(dblNumber : longint) : string;
Var
  strOut : string;
begin
  strOut := Chr(trunc(dblNumber / 16777216));
  strOut := strOut + Chr( trunc((dblNumber And 16711680) / 65536));
  strOut := strOut + Chr( trunc((dblNumber And 65280) / 256));
  strOut := strOut + Chr(dblNumber And 255);
  FourBytes:= strOut;
End;

Function TMainForm.TwoBytes(lngNumber : longint) : string;
Var
 strOut : string;
begin
    strOut := Chr( trunc((lngNumber And 65280) / 256));
    strOut := strOut + Chr(lngNumber And 255);
    TwoBytes := strOut;
End;


procedure TmainForm.B_selRepDestClick(Sender: TObject);
var
  dir : string;
begin

  Dir := self.repDestPDB.Text;
  if SelectDirectory(Dir, [sdAllowCreate, sdPerformCreate, sdPrompt],1000) then
         self.repDestPDB.Text := dir;

end;




procedure TmainForm.B_selFicSourceClick(Sender: TObject);
begin

if OpenDialog1.Execute then
  begin
   self.fichierSource.Text := OpenDialog1.FileName;
   self.StoryName.Text     := extractFileName(OpenDialog1.FileName);
  end;
end;

procedure TmainForm.B_transformerClick(Sender: TObject);
var
        resultat, texteAEcrire : string;
        memosource, lu : string;
        numFich : TextFILE;
begin

if (length(self.fichierSource.text)=0) or (length(self.StoryName.text)=0) or (length(self.repDestPDB.text)=0) then
        showMessage('Veuillez renseigner tous les champs.')
else
        begin

           // lire le fichier texte et le mmoriser
           assignFile(numFich, self.fichierSource.Text);
           reset(numFich);
           while not eof(numfich)do
           begin
             readln(numfich,lu);
             memosource:=memosource+lu+#10;
           end;
           closeFile(numfich);

           showmessage( intToStr( length(memosource)) + ' caractres lus.');

           // transformer le texte en PDB
           resultat:= StringReplace( memoSource,#13,'',[rfReplaceAll]);
           texteAEcrire:= Txt2Pdb( resultat, self.storyname.Text,1 );

           // sauvegarder le PDB
           assignFile(numFich, self.repDestPDB.Text+'\'+self.storyname.text+'.pdb');
           rewrite(numfich);
           write(numFich, texteAEcrire);
           closeFile(numFich);

           showmessage( intToStr( length(texteAecrire)) + ' caractres cris.');

           showmessage('Fichier '+self.repDestPDB.Text+'\'+self.storyname.text+'.pdb'+' sauvegard');

        end;

end;




procedure TmainForm.Button1Click(Sender: TObject);
begin

showmessage ('Programme de conversion de fichier texte vers pdb - dvelopp par Tactics - http://palmpascal.free.fr');

end;

end.
